#!/usr/bin/perl

my ($datei) = @ARGV;

chomp $datei;
open LIBELLE, "$datei" or die "\nfile in diesem Ordner nicht vorhanden!";
	@IN=<LIBELLE>;
	$IN=join("",@IN);

#SORTS STRUCTURE STRING__________________________________________________________________
#generates singles and pairs, from structure string, which are used for random sampling
#of characters taking care of character dependencies,
#uses subroutines "nesting", "sorting" and "singles_pairs"

	my $nested=nesting($IN);
	my $sorted=sorting($nested);
	my @structureinfo=split/\n/,$sorted;
	my $sortedstructurestring=$structureinfo[2];
	#print $sortedstructurestring,"\n";
	my $sites=singles_pairs($sorted);
			my 	$sortedpos=$sites;
				$sortedpos=~s/\:/\,/g;
			my	@sortedpos=split",",$sortedpos;

#_________________________________________________________________________________________

sub singles_pairs {
#PAIRS AND SINGLES
#delivers pairs and singles usable in MrBayes, for example, prints it out to file singles_pairs.txt
#needs as input first line of sub sorting output, sub sorting delivers three lines separated by \n, 1) sorted structure with nesting, site and dot bracket information, 2) sorted sites and 3) sorted dot-brackets
my (@input,$var,@pairs,$pairs2);
open OUT, ">singles_pairs.txt";
@input=@_;
	$var=join ("",@input);
	@pairs=split /\n/,$var;
	#print $pairs[0],"\n";
		$pairs2=$pairs[0];
		$pairs2=~s/(\d*)\-\d*\./$1\,/g;
		$pairs2=~s/(\d*)\-\d*\(/$1\:/g;
		$pairs2=~s/(\d*)\-\d*\)/$1\,/g;
chop $pairs2;
print OUT $pairs2;
close OUT;
#print $pairs2,"\n";
return $pairs2;
}

sub sorting {
#STRUCTURESORTING
#input file is a nested structure string, every site with its associated nesting information is separated by white spaces. 0. 0. 0( 1( 2( 3( 4( 4. 4. 4. 4. 4) 3) 2) 1) 0) 0. 0. 0. 0. 0. 0. 0. 0. 0. 0( 1( 2( 2. 2. 2. 2) 1) 0), the structure information follows the nesting level, everything is in one line
#the script is partitioned into three major units: 1) enumeration of positions, is important for the generation of the bootstrap lines later 2) pushing all dot sites into the structure array and afterwards the bracket sites 3) splitting the structure array into one sorted site string and one sorted dot-bracket string which are saved each in a new line in the OUT file
#open OUT, ">Structuresorted.txt";
#reads structure string and prepares it for further processing, can read structure string over multiple newlines correctly
my (@input,$var3,@structure,$n,@structurefinal,$level,$len,@first,@for,$p,$maxlevel);
	@input=@_;
	my $var=join ("",@input);
	#chomp $struc;
	@structure=split " ",$var;
	$n=@structure;
#delivers the $maxlevel as the maximamal nesting level
my @var3=@structure;
my @sorted=grep s/(\d*)\.*\(*\)*/$1/,@var3;
my @sorted2=sort{$a<=>$b}@sorted;
$maxlevel=$sorted2[-1];
#print @sorted,"\n",@sorted2,"\n",$maxlevel,"\n";
#enummerates sites and writes numbers into array elements, array elements contain information on site numbers, nesting level and structure
#$j is site number, $jmax equals sequence length of input file, or structure string, should be identical to $n, otherwise something is wrong
my $j=1;
my $bar="-";
for (my $i=0;$i<=$n;$i++){
	$var=shift @structure;
	my $var2=$j.$bar.$var;
	push @structurefinal,$var2;
	$j=$j+1;
	}
	pop @structurefinal;
#sorting of sites and structure information based on the structure nesting, first every non paired site is written into @structurefinal by pushing, afterwards a double loop is used in order to get the structure information into @structurefinal
$p="OK";
#refers to the nesting level
$level=0;
#refers to the length of the @structurefinal, should be: $n=$j-1=$len
$len=$#structurefinal;
		for (my $i=0;$i<=$len;$i++){
			@first=shift@structurefinal;
			if ($first[0]=~/\d+-.+\./){
				push @for, $first[0];
				push @structurefinal,$p;
				}
			else {
				push @structurefinal, $first[0];
				@first=();
				}
		}
#	print @for,"\n";
#	print @structurefinal,"\n";
#double loop of sorting
	for ($level=0;$level<=$maxlevel;$level++){
		for (my $i=0;$i<=$len;$i++){
			@first=shift@structurefinal;
			if ($first[0]=~/\d+\-${level}\(/){
				push @for, $first[0];
				push @structurefinal,$p;
				}
			elsif ($first[0]=~/\d+\-${level}\)/){
				push @for, $first[0];
				push @structurefinal,$p;
				}
			else {
				push @structurefinal, $first[0];
				@first=();
				}
			}
		}

#print "\n",@for,"\n";
#print "\n",@structurefinal,"\n";

my	@numbers=@for;
my	@dotbracket=@for;
my	@s1=grep s/(\d+)\-.*(\.*\(*\)*)/$1\,/,@numbers;
my	$s2=join ("",@s1);
my	@sitessorted= split//,$s2;
	pop @sitessorted;
my	@sorteddotbracket=grep s/\d+\-\d+(\.*\(*\)*)/$1/,@dotbracket;

#print "\n",@for,"\n","first sorted sites:\n",@sitessorted,"\n","now the dot bracket:\n",@sorteddotbracket,"\n";
#print OUT join ("",@for),"\n",join ("",@sitessorted),"\n",join ("",@sorteddotbracket),"\n";
#close OUT;
$var3= join ("",@for)."\n".join ("",@sitessorted)."\n".join ("",@sorteddotbracket)."\n";
return $var3;
}

sub nesting {
#STRUCTURENESTING
#input file is a dot bracket structure string, script delivers nesting information on structure strng separated by white spaces. 0. 0. 0( 1( 2( 3( 4( 4. 4. 4. 4. 4) 3) 2) 1) 0) 0. 0. 0. 0. 0. 0. 0. 0. 0. 0( 1( 2( 2. 2. 2. 2) 1) 0), the structure information follows the nesting level, final output is a string in the above form
my ($var,$level,$position,$positionnested1,$positionnested2,$positionnested3,@structure,@structurenested,$dim);
#open OUT, ">Dot-Brackets-nested.txt";
$var=$_[0];
$var=~s/(.)/$1 \,/g;
chomp $var;
@structure=split ",",$var;
$dim=@structure;
$level=0;
$position=();
for (my $i=0;$i<=$dim;$i++){
	my $position= shift @structure;
	if ($position=~/\. /){
		my $space=" ";
		$positionnested1=$level.$position.$space;
		push (@structurenested,$positionnested1);
		}
	elsif ($position=~/\( /){
		$level=$level+1;
		$positionnested2=$level.$position." ";
		push (@structurenested,$positionnested2);
		}
	elsif ($position=~/\) /){
		$positionnested3=$level.$position." ";
		push (@structurenested,$positionnested3);
		$level=$level-1;
		$position=();
		}
	}

#print @structurenested,"\n";
#print OUT join ("",@structurenested);
#print "Length of string: $dim\n";
#close OUT;
return $var= join ("",@structurenested);
}

exit;